# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import os
from django.conf import settings
from django.utils._os import upath
from django.utils.importlib import import_module
from django.test.signals import setting_changed
import django.utils.translation.trans_real
from po_localization.file_watcher import FileWatcher
from po_localization.translations_loader import TranslationsLoader
from po_localization.translations_updater import TranslationsUpdater


class PoLocalizationMiddleware(object):
    def __init__(self):
        self.translations_updater = TranslationsUpdater()
        self.translations_loader = TranslationsLoader()
        self.translations_updater_watcher = FileWatcher(self.translations_updater)
        self.translations_loader_watcher = FileWatcher(self.translations_loader)
        self.reconfigure()
        setting_changed.connect(self._reconfigure)

    def _reconfigure(self, sender, **kwargs):
        self.reconfigure()

    def reconfigure(self):
        self.translations_updater.root_paths = get_translations_update_roots()
        self.translations_updater.locales = get_enabled_locales(
            excluded_locales=getattr(settings, 'UPDATE_TRANSLATIONS_EXCLUDED_LOCALES', ()))
        self.translations_updater.include_locations = getattr(settings, 'UPDATE_TRANSLATIONS_WITH_LOCATIONS', True)
        self.translations_updater.prune_obsoletes = getattr(settings, 'UPDATE_TRANSLATIONS_PRUNE_OBSOLETES', False)
        # Force update in case any setting has changed (which changes the output)
        self.translations_updater_watcher.set_dirty()

        self.translations_loader.locales = get_enabled_locales()
        self.translations_loader.locale_paths = get_translations_reload_roots()

    def process_request(self, request):
        if getattr(settings, 'AUTO_UPDATE_TRANSLATIONS', False):
            self.translations_updater_watcher.check()
        if getattr(settings, 'AUTO_RELOAD_TRANSLATIONS', settings.DEBUG):
            self.translations_loader_watcher.check()


def get_enabled_locales(excluded_locales=()):
    ret = []
    for language_code, language_name in settings.LANGUAGES:
        locale = django.utils.translation.trans_real.to_locale(language_code)
        if locale not in excluded_locales:
            ret.append(locale)
    return ret


def get_translations_reload_roots():
    ret = []
    localization_packages = ['django.conf']
    # TODO: use app registry in django >= 1.7
    localization_packages.extend(reversed(settings.INSTALLED_APPS))
    ret.extend(get_packages_paths(localization_packages, 'locale'))
    for locale_path in reversed(settings.LOCALE_PATHS):
        if os.path.isdir(locale_path):
            ret.append(locale_path)
    return ret


def get_translations_update_roots():
    return get_packages_paths(getattr(settings, 'UPDATE_TRANSLATIONS_PACKAGES', ()))


def get_packages_paths(packages_import_paths, relative_path=''):
    ret = []
    for package_import_path in packages_import_paths:
        module = import_module(package_import_path)
        module_path = os.path.dirname(upath(module.__file__))
        ret.append(os.path.join(module_path, relative_path))
    return ret



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from django.conf.urls import url
from .views import test_view

urlpatterns = [
    url(r'^$', test_view)
]



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import ast
import io

GETTEXT_FUNCTIONS = ('gettext', 'gettext_lazy', 'gettext_noop', 'ugettext', 'ugettext_lazy', 'ugettext_noop')
PGETTEXT_FUNCTIONS = ('pgettext', 'pgettext_lazy')
NGETTEXT_FUNCTIONS = ('ngettext', 'ungettext', 'ngettext_lazy', 'ungettext_lazy')
NPGETTEXT_FUNCTIONS = ('npgettext', 'npgettext_lazy')


def extract_messages(filename, po_file, printable_filename=None):
    printable_filename = filename if printable_filename is None else printable_filename
    with io.open(filename, mode='rb') as python_file:
        file_content = python_file.read()
    tree = ast.parse(file_content, printable_filename)
    extractor = PythonExtractor(printable_filename, po_file)
    extractor.visit(tree)


class PythonExtractor(ast.NodeVisitor):
    def __init__(self, filename, po_file):
        self.filename = filename
        self.aliases = ScopedAliases()
        self.po_file = po_file

    def visit_Import(self, node):
        for alias in node.names:
            if alias.name == 'django.utils.translation':
                self._add_function_aliases_for_module(alias)

    def visit_ImportFrom(self, node):
        if node.level == 0:
            if node.module == 'django.utils.translation':
                for alias in node.names:
                    visible_name = alias.name if alias.asname is None else alias.asname
                    if alias.name in GETTEXT_FUNCTIONS:
                        self.aliases.set(visible_name, 'gettext')
                    elif alias.name in PGETTEXT_FUNCTIONS:
                        self.aliases.set(visible_name, 'pgettext')
                    elif alias.name in NGETTEXT_FUNCTIONS:
                        self.aliases.set(visible_name, 'ngettext')
                    elif alias.name in NPGETTEXT_FUNCTIONS:
                        self.aliases.set(visible_name, 'npgettext')
            elif node.module == 'django.utils':
                for alias in node.names:
                    if alias.name == 'translation':
                        self._add_function_aliases_for_module(alias)

    def _add_function_aliases_for_module(self, module_alias):
        visible_name = module_alias.name if module_alias.asname is None else module_alias.asname
        for function in GETTEXT_FUNCTIONS:
            self.aliases.set(visible_name + '.' + function, 'gettext')
        for function in PGETTEXT_FUNCTIONS:
            self.aliases.set(visible_name + '.' + function, 'pgettext')
        for function in NGETTEXT_FUNCTIONS:
            self.aliases.set(visible_name + '.' + function, 'ngettext')
        for function in NPGETTEXT_FUNCTIONS:
            self.aliases.set(visible_name + '.' + function, 'npgettext')

    def visit_FunctionDef(self, node):
        self.aliases = ScopedAliases(self.aliases)
        self.generic_visit(node)
        self.aliases = self.aliases.parent_scope

    def visit_Call(self, node):
        function_name = _get_dotted_name(node.func)
        aliased_name = self.aliases.get(function_name)
        if aliased_name is not None:
            entry = None
            # XXX: args could be passed in *args, **kwargs or as keywords
            if aliased_name == 'gettext' and _first_args_are_strings(node.args, 1):
                entry = self.po_file.add_entry(message=node.args[0].s)
            elif aliased_name == 'pgettext' and _first_args_are_strings(node.args, 2):
                entry = self.po_file.add_entry(context=node.args[0].s, message=node.args[1].s)
            elif aliased_name == 'ngettext' and _first_args_are_strings(node.args, 1):
                entry = self.po_file.add_entry(message=node.args[0].s, plural=node.args[1].s)
            elif aliased_name == 'npgettext' and _first_args_are_strings(node.args, 2):
                entry = self.po_file.add_entry(context=node.args[0].s, message=node.args[1].s, plural=node.args[2].s)
            if entry:
                entry.add_location(self.filename, node.lineno)
        self.generic_visit(node)

    # The rest of the code is here only for performances reasons
    # you can remove it and keep exactly the same visible behavior.

    # These are sorted by frequency
    IGNORED_CLASSES = (ast.Name, str, ast.Load, ast.Str, ast.Num, bool)

    def generic_visit(self, node):
        for field in node._fields:
            value = getattr(node, field)
            if value.__class__ is list:
                for item in value:
                    if item is not None and item.__class__ not in self.IGNORED_CLASSES:
                        self.visit(item)
            elif value is not None and value.__class__ not in self.IGNORED_CLASSES:
                self.visit(value)

    def visit(self, node):
        cls = node.__class__
        if cls is ast.Call:
            self.visit_Call(node)
        elif cls is ast.Import:
            self.visit_Import(node)
        elif cls is ast.ImportFrom:
            self.visit_ImportFrom(node)
        elif cls is ast.FunctionDef:
            self.visit_FunctionDef(node)
        else:
            self.generic_visit(node)


class ScopedAliases(object):
    def __init__(self, parent_scope=None):
        self.aliases = {}
        self.parent_scope = parent_scope

    def get(self, name):
        aliased_name = self.aliases.get(name, None)
        if aliased_name is None and self.parent_scope is not None:
            return self.parent_scope.get(name)
        else:
            return aliased_name

    def set(self, name, aliased_name):
        self.aliases[name] = aliased_name


def _first_args_are_strings(args, length):
    return len(args) >= length and all(isinstance(arg, ast.Str) for arg in args[:length])


def _get_dotted_name(node, suffix=''):
    if isinstance(node, ast.Name):
        return node.id + suffix
    elif isinstance(node, ast.Attribute):
        return _get_dotted_name(node.value, '.' + node.attr + suffix)
    else:
        return None



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import os
import sys
import threading


class FileWatcher(object):
    def __init__(self, operator):
        self.operator = operator
        self.file_mtimes = {}
        self.is_dirty = True
        self.lock = threading.Lock()

    def set_dirty(self):
        self.is_dirty = True

    def check(self):
        with self.lock:
            self._check_for_changes()
            if self.is_dirty:
                self.operator.execute()
                self.is_dirty = False

    def _check_for_changes(self):
        files_list = set(self.operator.list_files())
        for file_path in list(self.file_mtimes.keys()):
            if file_path not in files_list:
                self.is_dirty = True
                del self.file_mtimes[file_path]
        for file_path in files_list:
            if os.path.isfile(file_path):
                file_mtime = get_file_mtime(file_path)
                if file_path not in self.file_mtimes or self.file_mtimes[file_path] != file_mtime:
                    self.file_mtimes[file_path] = file_mtime
                    self.is_dirty = True
            elif file_path in self.file_mtimes:
                self.is_dirty = True
                del self.file_mtimes[file_path]


def get_file_mtime(filename):
    stat = os.stat(filename)
    mtime = stat.st_mtime
    if sys.platform == 'win32':
        mtime -= stat.st_ctime
    return mtime



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import base64
from importlib import import_module
import os
import pickle
import sys
from po_localization.tests.subtest import PicklableTestResult, print_bytes

if __name__ == '__main__':
    if 'COVERAGE_PROCESS_START' in os.environ:
        import coverage
        coverage.process_startup()
    module_name, class_name, method_name = sys.argv[1:]
    module = import_module(module_name)
    test_case_class = getattr(module, class_name)
    test_case = test_case_class(method_name)
    result = PicklableTestResult()
    test_case(result)
    pickled_result = pickle.dumps(result.storage, protocol=pickle.HIGHEST_PROTOCOL)
    encoded_result = base64.encodestring(pickled_result)
    print_bytes(b'-')
    print_bytes(encoded_result)



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import re
import sys

# PyCharm incorrectly flags \v as an invalid escape sequence in python regexp, hence the \x0b
ESCAPER = re.compile(r'[\a\b\f\n\r\t\x0b\\"]')
UNESCAPER = re.compile(r'\\(?:([abfnrtv\\"])|([0-7]+)|x([0-9a-fA-F]+)|(.))?')

unicode_chr = unichr if sys.version_info[0] == 2 else chr


def escape(string):
    return ESCAPER.sub(_escape_char, string)


def _escape_char(match):
    char = match.group(0)
    if char == '\a':
        return r'\a'
    elif char == '\b':
        return r'\b'
    elif char == '\f':
        return r'\f'
    elif char == '\n':
        return r'\n'
    elif char == '\r':
        return r'\r'
    elif char == '\t':
        return r'\t'
    elif char == '\v':
        return r'\v'
    elif char == '\\':
        return r'\\'
    else:
        return r'\"'


def unescape(escaped_string):
    return UNESCAPER.sub(_unescape_code, escaped_string)


def _unescape_code(match):
    if match.group(1) is not None:
        code = match.group(1)
        if code == 'a':
            return '\a'
        elif code == 'b':
            return '\b'
        elif code == 'f':
            return '\f'
        elif code == 'n':
            return '\n'
        elif code == 'r':
            return '\r'
        elif code == 't':
            return '\t'
        elif code == 'v':
            return '\v'
        else:
            return code
    elif match.group(2) is not None:
        return unicode_chr(int(match.group(2), base=8))
    elif match.group(3) is not None:
        return unicode_chr(int(match.group(3), base=16))
    else:
        raise UnescapeError("Invalid escape sequence: \\{}".format(match.group(4)))


class UnescapeError(Exception):
    pass



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from django.http import HttpResponse
from django.utils.translation import ugettext as _


def test_view(request):
    return HttpResponse(_('test project string'))



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from unittest import TestCase
from po_localization.strings import escape, unescape, UnescapeError


class EscapeTextCase(TestCase):
    def test_empty(self):
        self.assertEqual("", escape(""))

    def test_simple(self):
        self.assertEqual(r"First\nSecond", escape("First\nSecond"))

    def test_multiple(self):
        self.assertEqual(r"\a \b \f \n \r \t \v \\ \"", escape("\a \b \f \n \r \t \v \\ \""))


class UnescapeTestCase(TestCase):
    def test_empty(self):
        self.assertEqual("", unescape(""))

    def test_simple(self):
        self.assertEqual("First\nSecond", unescape(r"First\nSecond"))

    def test_multiple(self):
        self.assertEqual("\a \b \f \n \r \t \v \\ \"", unescape(r"\a \b \f \n \r \t \v \\ \""))

    def test_octal_escape(self):
        self.assertEqual("e", unescape(r"\145"))
        self.assertEqual("€", unescape(r"\20254"))

    def test_hexadecimal_escape(self):
        self.assertEqual("e", unescape(r"\x65"))
        self.assertEqual("€", unescape(r"\x20ac"))
        self.assertEqual("€", unescape(r"\x20AC"))
        self.assertEqual("€", unescape(r"\x20aC"))

    def test_unfinished_escape(self):
        self.assertRaises(UnescapeError, unescape, "\\")

    def test_invalid_escape(self):
        self.assertRaises(UnescapeError, unescape, "\\FAIL")



# coding=utf-8

import django.utils.translation
import django.utils.translation as translation_module
from django.utils import translation as translation_module_2
from django.utils.translation import (
    gettext as _,
    gettext_lazy as _l,
    gettext_noop as _0x90,
    ugettext as _u,
    ugettext_lazy as _ul,
    ugettext_noop as _u0x90,
    pgettext as _p,
    pgettext_lazy as _pl,
    ngettext as _n,
    ngettext_lazy as _nl,
    ungettext as _un,
    ungettext_lazy as _unl,
    npgettext as _np,
    npgettext_lazy as _npl,
)

django.utils.translation.gettext('test')
django.utils.translation.gettext_lazy('test_lazy')
django.utils.translation.gettext_noop('test_noop')
django.utils.translation.ugettext('utest')
django.utils.translation.ugettext_lazy('utest_lazy')
django.utils.translation.ugettext_noop('utest_noop')
django.utils.translation.pgettext('context', 'ptest')
django.utils.translation.pgettext_lazy('context', 'ptest_lazy')
django.utils.translation.ngettext('ntest', 'ntests', 42)
django.utils.translation.ngettext_lazy('ntest_lazy', 'ntests_lazy', 42)
django.utils.translation.ungettext('untest', 'untests', 42)
django.utils.translation.ungettext_lazy('untest_lazy', 'untests_lazy', 42)
django.utils.translation.npgettext('context', 'nptest', 'nptests', 42)
django.utils.translation.npgettext_lazy('context', 'nptest_lazy', 'nptests_lazy', 42)

translation_module.gettext('tm_test')
translation_module.gettext_lazy('tm_test_lazy')
translation_module.gettext_noop('tm_test_noop')
translation_module.ugettext('tm_utest')
translation_module.ugettext_lazy('tm_utest_lazy')
translation_module.ugettext_noop('tm_utest_noop')
translation_module.pgettext('context', 'tm_ptest')
translation_module.pgettext_lazy('context', 'tm_ptest_lazy')
translation_module.ngettext('tm_ntest', 'tm_ntests', 42)
translation_module.ngettext_lazy('tm_ntest_lazy', 'tm_ntests_lazy', 42)
translation_module.ungettext('tm_untest', 'tm_untests', 42)
translation_module.ungettext_lazy('tm_untest_lazy', 'tm_untests_lazy', 42)
translation_module.npgettext('context', 'tm_nptest', 'tm_nptests', 42)
translation_module.npgettext_lazy('context', 'tm_nptest_lazy', 'tm_nptests_lazy', 42)

translation_module_2.gettext('tm2_test')
translation_module_2.gettext_lazy('tm2_test_lazy')
translation_module_2.gettext_noop('tm2_test_noop')
translation_module_2.ugettext('tm2_utest')
translation_module_2.ugettext_lazy('tm2_utest_lazy')
translation_module_2.ugettext_noop('tm2_utest_noop')
translation_module_2.pgettext('context', 'tm2_ptest')
translation_module_2.pgettext_lazy('context', 'tm2_ptest_lazy')
translation_module_2.ngettext('tm2_ntest', 'tm2_ntests', 42)
translation_module_2.ngettext_lazy('tm2_ntest_lazy', 'tm2_ntests_lazy', 42)
translation_module_2.ungettext('tm2_untest', 'tm2_untests', 42)
translation_module_2.ungettext_lazy('tm2_untest_lazy', 'tm2_untests_lazy', 42)
translation_module_2.npgettext('context', 'tm2_nptest', 'tm2_nptests', 42)
translation_module_2.npgettext_lazy('context', 'tm2_nptest_lazy', 'tm2_nptests_lazy', 42)

_('alias_test')
_l('alias_test_lazy')
_0x90('alias_test_noop')
_u('alias_utest')
_ul('alias_utest_lazy')
_u0x90('alias_utest_noop')
_p('context', 'alias_ptest')
_pl('context', 'alias_ptest_lazy')
_n('alias_ntest', 'alias_ntests', 42)
_nl('alias_ntest_lazy', 'alias_ntests_lazy', 42)
_un('alias_untest', 'alias_untests', 42)
_unl('alias_untest_lazy', 'alias_untests_lazy', 42)
_np('context', 'alias_nptest', 'alias_nptests', 42)
_npl('context', 'alias_nptest_lazy', 'alias_nptests_lazy', 42)


def gettext(message):
    pass


gettext('not a translation')


def some_function():
    from django.utils.translation import gettext

    gettext('a translation')


gettext('still not a translation')


def something_else():
    pass


something_else().gettext('not a translation')

something = {}
something["else"].gettext('not a translation')




# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Update translation files'

    def handle(self, *args, **options):
        from ...middleware import PoLocalizationMiddleware
        po_localization_middleware = PoLocalizationMiddleware()
        po_localization_middleware.translations_updater.execute()



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import os
from io import StringIO
from unittest import TestCase
from po_localization.parser import Parser, ParseError, parse_po_file, parse_po_filename
from po_localization.po_file import PoFile

class ParserTestCase(TestCase):
    def _parse_and_expect(self, file_content, expected_catalog):
        file_object = StringIO(file_content)
        self.assertDictEqual(expected_catalog, parse_po_file(file_object))

    def _parse_and_expect_failure(self, file_content):
        file_content = StringIO(file_content)
        self.assertRaises(ParseError, parse_po_file, file_content)

    def test_empty_file(self):
        self._parse_and_expect("", {})

    def test_comment(self):
        self._parse_and_expect("""
# Comment
""", {})

    def test_empty_line(self):
        self._parse_and_expect("""

""", {})

    def test_simple(self):
        self._parse_and_expect("""
msgid "Message to translate"
msgstr "Message translated"
""", {
            "Message to translate": "Message translated"
        })

    def test_unescaping(self):
        self._parse_and_expect(r"""
msgid "Line One\nLine Two"
msgstr "First Line\nSecond Line"
""", {
            "Line One\nLine Two": "First Line\nSecond Line"
        })

    def test_broken_unescaping(self):
        self._parse_and_expect_failure(r"""
msgid "Line One\"
"nLine Two"
msgstr "First Line\nSecond Line"
""")

    def test_header_not_in_catalog(self):
        self._parse_and_expect(r"""
msgid ""
msgstr ""
"Project-Id-Version: Django\n"

msgid "Message to translate"
msgstr "Message translated"
""", {
            "Message to translate": "Message translated"
        })

    def test_context(self):
        self._parse_and_expect("""
msgctxt "Context"
msgid "Message to translate"
msgstr "Translated message"
""", {
            "Context\x04Message to translate": "Translated message"
        })

    def test_plural(self):
        self._parse_and_expect("""
msgid "Message to translate"
msgid_plural "Messages to translate"
msgstr[0] "Translated message"
msgstr[1] "Translated messages"
""", {
            ("Message to translate", 0): "Translated message",
            ("Message to translate", 1): "Translated messages"
        })

    def test_context_and_plural(self):
        self._parse_and_expect("""
msgctxt "Context"
msgid "Message to translate"
msgid_plural "Messages to translate"
msgstr[0] "Translated message"
msgstr[1] "Translated messages"
""",
            {
                ("Context\x04Message to translate", 0): "Translated message",
                ("Context\x04Message to translate", 1): "Translated messages"
            })

    def test_unexpected_keywords(self):
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgctxt "Context"
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgctxt "Context"
msgstr "Translated message"
msgid "Message to translate"
""")
        self._parse_and_expect_failure("""
poney "Poney"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgid_plural "Messages to translate"
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgid_plural "Messages to translate"
msgstr[0] "Translated message"
msgstr "Translated messages"
""")

    def test_duplicate_keywords(self):
        self._parse_and_expect_failure("""
msgctxt "Context"
msgctxt "Context"
msgid "Message to translate"
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgctxt "Context"
msgid "Message to translate"
msgid "Message to translate"
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgctxt "Context"
msgid "Message to translate"
msgstr "Translated message"
msgstr "Translated message"
""")

    def test_duplicate_plural_index(self):
        self._parse_and_expect_failure("""
msgctxt "Context"
msgid "Message to translate"
msgid_plural "Messages to translate"
msgstr[0] "Translated message"
msgstr[0] "Translated message again"
""")

    def test_early_termination(self):
        self._parse_and_expect_failure("""
msgctxt "Context"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgid_plural "Messages to translate"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
"... with continuation"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
# With comment
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"

""")

    def test_missing_string(self):
        self._parse_and_expect_failure("""
msgctxt
msgid "Message to translate"
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgid
msgstr "Translated Message"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgstr
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgid_plural
msgstr[0] "Translated message"
msgstr[1] "Translated messages"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgid_plural "Messages to translate"
msgstr[0]
msgstr[1] "Translated messages"
""")

    def test_partial_string(self):
        self._parse_and_expect_failure("""
msgctxt "Context
msgid "Message to translate"
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgstr "Translated message
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgid_plural "Messages to translate
msgstr[0] "Translated message"
msgstr[1] "Translated messages"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgid_plural "Messages to translate"
msgstr[0] "Translated message
msgstr[1] "Translated messages"
""")

    def test_unexpected_index(self):
        self._parse_and_expect_failure("""
msgctxt[0] "Context"
msgid "Message to translate"
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgctxt "Context"
msgid[0] "Message to translate"
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgctxt "Context"
msgid "Message to translate"
msgstr[0] "Translated message"
""")
        self._parse_and_expect_failure("""
msgctxt "Context"
msgid "Message to translate"
msgid_plural[0] "Messages to translate"
msgstr[0] "Translated message"
msgstr[1] "Translated messages"
""")

    def test_unexpected_continuation(self):
        self._parse_and_expect_failure("""
"Continuation"
""")
        self._parse_and_expect_failure("""
msgctxt "context"
"Continuation"
""")

    def test_garbage_at_end(self):
        self._parse_and_expect_failure("""
msgctxt "Context" GARBAGE
msgid "Message to translate"
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate" GARBAGE
msgstr "Translated message"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgstr "Translated message" GARBAGE
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgid_plural "Messages to translate" GARBAGE
msgstr[0] "Translated message"
msgstr[1] "Translated messages"
""")
        self._parse_and_expect_failure("""
msgid "Message to translate"
msgid_plural "Messages to translate"
msgstr[0] "Translated message" GARBAGE
msgstr[1] "Translated messages"
""")

    def test_real_file(self):
        filename = os.path.join(os.path.dirname(__file__), 'sample.po')
        self.assertDictEqual(parse_po_filename(filename), {
            ("Context\x04Message to translate", 0): "Message à traduire",
            ("Context\x04Message to translate", 1): "Messages à traduire",
        })

    def test_header_parsing(self):
        file_object = StringIO(r"""
msgid ""
msgstr ""
"Project-Id-Version: Django\n"
"Report-Msgid-Bugs-To: \n"
"Language-Team: French <None>\n"
"Language: fr\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=UTF-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Plural-Forms: nplurals=2; plural=(n > 1)\n"
""")
        po_file = PoFile()
        self.assertListEqual(po_file.header_fields, [])
        self.assertIsNone(po_file.get_nplurals())
        parser = Parser(po_file)
        parser.parse_po_file(file_object)
        self.assertListEqual(po_file.header_fields, [
            ('Project-Id-Version', 'Django'),
            ('Report-Msgid-Bugs-To', ''),
            ('Language-Team', 'French <None>'),
            ('Language', 'fr'),
            ('MIME-Version', '1.0'),
            ('Content-Type', 'text/plain; charset=UTF-8'),
            ('Content-Transfer-Encoding', '8bit'),
            ('Plural-Forms', 'nplurals=2; plural=(n > 1)')])
        self.assertEqual(po_file.get_nplurals(), 2)

    def test_exception_message(self):
        try:
            raise ParseError("filename.po", 42, "the error message")
        except ParseError as error:
            self.assertEqual("filename.po:42: the error message", "{}".format(error))

        try:
            raise ParseError("filename.po", None, "unexpected end of file")
        except ParseError as error:
            self.assertEqual("filename.po: unexpected end of file", "{}".format(error))

        try:
            raise ParseError(None, 42, "the error message")
        except ParseError as error:
            self.assertEqual("line 42: the error message", "{}".format(error))

        try:
            raise ParseError(None, None, "the error message")
        except ParseError as error:
            self.assertEqual("the error message", "{}".format(error))




# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import os
import shutil
import tempfile
import time
from unittest import TestCase
from po_localization.file_watcher import FileWatcher


class FileWatcherTestCase(TestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_empty(self):
        operator = TestOperator()
        file_watcher = FileWatcher(operator)
        file_watcher.check()
        self.assertEqual(1, operator.list_files_calls)
        self.assertEqual(1, operator.execute_calls)
        file_watcher.check()
        self.assertEqual(2, operator.list_files_calls)
        self.assertEqual(1, operator.execute_calls)
        file_watcher.set_dirty()
        self.assertEqual(2, operator.list_files_calls)
        self.assertEqual(1, operator.execute_calls)
        file_watcher.check()
        self.assertEqual(3, operator.list_files_calls)
        self.assertEqual(2, operator.execute_calls)
        file_watcher.check()
        self.assertEqual(4, operator.list_files_calls)
        self.assertEqual(2, operator.execute_calls)

    def test_add_file_to_list(self):
        file_path = os.path.join(self.temp_dir, 'file.ext')
        with open(file_path, 'w'):
            pass
        operator = TestOperator()
        file_watcher = FileWatcher(operator)
        file_watcher.check()
        self.assertEqual(1, operator.list_files_calls)
        self.assertEqual(1, operator.execute_calls)
        operator.files_list = (__file__,)
        file_watcher.check()
        self.assertEqual(2, operator.list_files_calls)
        self.assertEqual(2, operator.execute_calls)

    def test_touch_file(self):
        file_path = os.path.join(self.temp_dir, 'file.ext')
        with open(file_path, 'w'):
            pass
        start_time = time.time()
        operator = TestOperator((file_path,))
        os.utime(file_path, (0, start_time))
        file_watcher = FileWatcher(operator)
        file_watcher.check()
        self.assertEqual(1, operator.list_files_calls)
        self.assertEqual(1, operator.execute_calls)
        file_watcher.check()
        self.assertEqual(2, operator.list_files_calls)
        self.assertEqual(1, operator.execute_calls)
        os.utime(file_path, (0, start_time + 1))
        file_watcher.check()
        self.assertEqual(3, operator.list_files_calls)
        self.assertEqual(2, operator.execute_calls)

    def test_remove_file(self):
        file_path = os.path.join(self.temp_dir, 'file.ext')
        with open(file_path, 'w'):
            pass
        operator = TestOperator((file_path,))
        file_watcher = FileWatcher(operator)
        file_watcher.check()
        self.assertEqual(1, operator.list_files_calls)
        self.assertEqual(1, operator.execute_calls)
        os.unlink(file_path)
        file_watcher.check()
        self.assertEqual(2, operator.list_files_calls)
        self.assertEqual(2, operator.execute_calls)

    def test_remove_file_from_list(self):
        file_path = os.path.join(self.temp_dir, 'file.ext')
        with open(file_path, 'w'):
            pass
        operator = TestOperator((file_path,))
        file_watcher = FileWatcher(operator)
        file_watcher.check()
        self.assertEqual(1, operator.list_files_calls)
        self.assertEqual(1, operator.execute_calls)
        operator.files_list = ()
        file_watcher.check()
        self.assertEqual(2, operator.list_files_calls)
        self.assertEqual(2, operator.execute_calls)


class TestOperator(object):
    def __init__(self, files_list=()):
        self.files_list = files_list
        self.execute_calls = 0
        self.list_files_calls = 0

    def execute(self):
        self.execute_calls += 1

    def list_files(self):
        self.list_files_calls += 1
        return self.files_list



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from unittest import TestCase
from po_localization.po_file import PoFile


class PoFileTestCase(TestCase):
    def test_message(self):
        po_file = PoFile()
        entry = po_file.add_entry("message")
        self.assertEqual("message", entry.message)
        self.assertIn("message", po_file.entries)
        self.assertIs(entry, po_file.entries["message"])

    def test_message_with_plural(self):
        po_file = PoFile()
        entry = po_file.add_entry("message", "plural message")
        self.assertEqual("message", entry.message)
        self.assertEqual("plural message", entry.plural)
        self.assertIn("message", po_file.entries)
        self.assertIs(entry, po_file.entries["message"])

    def test_message_with_context(self):
        po_file = PoFile()
        entry = po_file.add_entry("message", context="context")
        self.assertEqual("message", entry.message)
        self.assertEqual("context", entry.context)
        self.assertIn("context\x04message", po_file.entries)
        self.assertIs(entry, po_file.entries["context\x04message"])

    def test_translation(self):
        po_file = PoFile()
        entry = po_file.add_entry("message")
        entry.add_translation("translation")
        self.assertEqual({0: "translation"}, entry.translations)
        self.assertEqual({"message": "translation"}, po_file.get_catalog())

    def test_plural_translation(self):
        po_file = PoFile()
        entry = po_file.add_entry("message", "plural")
        entry.add_plural_translation(0, "translation")
        entry.add_plural_translation(1, "plural translation")
        self.assertEqual({0: "translation", 1: "plural translation"}, entry.translations)
        self.assertEqual(
            {
                ("message", 0): "translation",
                ("message", 1): "plural translation"},
            po_file.get_catalog())

    def test_location(self):
        po_file = PoFile()
        entry = po_file.add_entry("message")
        entry.add_location("filename.py", 42)
        entry.add_location("filename.py", 4242)
        self.assertEqual([("filename.py", 42), ("filename.py", 4242)], entry.locations)

    def test_header(self):
        po_file = PoFile()
        po_file.add_header_field("Language", "fr")
        po_file.add_header_field("MIME-Version", "1.0")
        self.assertEqual([("Language", "fr"), ("MIME-Version", "1.0")], po_file.header_fields)

    def test_header_overwrite(self):
        po_file = PoFile()
        po_file.add_header_field("Language", "fr")
        po_file.add_header_field("Language", "en")
        self.assertEqual([("Language", "en")], po_file.header_fields)

    def test_nplurals(self):
        po_file = PoFile()
        po_file.add_header_field("Plural-Forms", "nplurals=2; plural=(n > 1)")
        self.assertEqual(2, po_file.get_nplurals())

    def test_catalog(self):
        po_file = PoFile()
        po_file.add_entry("non translated message")
        po_file.add_entry("translated message").add_translation("translation")
        entry = po_file.add_entry("translated message 2", "plural")
        entry.add_plural_translation(0, "translation 2")
        entry.add_plural_translation(1, "plural translation 2")
        po_file.add_entry("translated message 3", context="context").add_translation("translation 3")
        entry = po_file.add_entry("translated message 4", "plural", "context")
        entry.add_plural_translation(0, "translation 4")
        entry.add_plural_translation(1, "plural translation 4")
        self.assertEqual(
            {
                "translated message": "translation",
                ("translated message 2", 0): "translation 2",
                ("translated message 2", 1): "plural translation 2",
                "context\x04translated message 3": "translation 3",
                ("context\x04translated message 4", 0): "translation 4",
                ("context\x04translated message 4", 1): "plural translation 4"},
            po_file.get_catalog())

    def test_dump(self):
        po_file = PoFile()
        po_file.add_entry("non translated message")
        po_file.add_entry("non translated message with plural", "plural")
        po_file.add_entry("translated message").add_translation("translation")
        entry_2 = po_file.add_entry("translated message 2", "plural")
        entry_2.add_plural_translation(0, "translation 2")
        entry_2.add_plural_translation(1, "plural translation 2")
        po_file.add_entry("translated message 3", context="context").add_translation("translation 3")
        entry_4 = po_file.add_entry("translated message 4", "plural", "context")
        entry_4.add_plural_translation(0, "translation 4")
        entry_4.add_plural_translation(1, "plural translation 4")
        entry_5 = po_file.add_entry("translated message 5")
        entry_5.add_location("filename.py", 42)
        entry_5.add_location("filename.py", 4242)
        self.assertEqual("""#. obsolete entry
msgid "translated message"
msgstr "translation"

#. obsolete entry
msgid "translated message 2"
msgid_plural "plural"
msgstr[0] "translation 2"
msgstr[1] "plural translation 2"

#. obsolete entry
msgctxt "context"
msgid "translated message 3"
msgstr "translation 3"

#. obsolete entry
msgctxt "context"
msgid "translated message 4"
msgid_plural "plural"
msgstr[0] "translation 4"
msgstr[1] "plural translation 4"

msgid "translated message 5"
msgstr ""
""", po_file.dumps(include_locations=False, prune_obsoletes=False))
        self.assertEqual("""msgid "translated message 5"
msgstr ""
""", po_file.dumps(include_locations=False, prune_obsoletes=True))
        self.assertEqual("""#: filename.py:42 filename.py:4242
msgid "translated message 5"
msgstr ""
""", po_file.dumps(include_locations=True, prune_obsoletes=True))

    def test_dump_nplurals(self):
        po_file = PoFile()
        entry = po_file.add_entry("message", "plural")
        entry.add_location("filename.py", 42)
        self.assertEqual("""msgid "message"
msgid_plural "plural"
msgstr[0] ""
msgstr[1] ""
""", po_file.dumps(include_locations=False, prune_obsoletes=False))
        po_file.add_header_field("Plural-Forms", "nplurals=3; plural=(n==1) ? 0 : (n>=2 && n<=4) ? 1 : 2;")
        self.assertEqual("""msgid ""
msgstr ""
"Plural-Forms: nplurals=3; plural=(n==1) ? 0 : (n>=2 && n<=4) ? 1 : 2;\\n"

msgid "message"
msgid_plural "plural"
msgstr[0] ""
msgstr[1] ""
msgstr[2] ""
""", po_file.dumps(include_locations=False, prune_obsoletes=False))



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import io
import re
from po_localization.po_file import PoFile
from .strings import unescape, UnescapeError

MATCHER = re.compile(r'^\s*(#.*)?\s*(?:([^"\[\s]+)(?:\[(\d+)\])?)?(?:\s*"(.*)")?\s*$')


def parse_po_filename(filename):
    po_file = PoFile()
    Parser(po_file).parse_po_filename(filename)
    return po_file.get_catalog()


def parse_po_file(fp, filename=None):
    po_file = PoFile()
    Parser(po_file).parse_po_file(fp)
    return po_file.get_catalog()


class Parser(object):
    def __init__(self, po_file):
        self.state = 'start'
        self.po_file = po_file
        self.context = None
        self.message = None
        self.plural_message = None
        self.translated_message = None
        self.current_plural_index = 0
        self.plural_translated_messages = {}

    def _store_pending_data(self):
        if self.message is not None:
            if self.message == '':
                for line in self.translated_message.split('\n'):
                    if len(line):
                        parts = line.partition(':')
                        self.po_file.add_header_field(parts[0], parts[2].strip())
            else:
                entry = self.po_file.add_entry(self.message, self.plural_message, self.context)
                if self.state == 'after msgstr':
                    entry.add_translation(self.translated_message)
                elif self.state == 'after msgstr[N]':
                    for index, text in self.plural_translated_messages.items():
                        entry.add_plural_translation(index, text)
        self._reset_pending_data()

    def _reset_pending_data(self):
        self.context = None
        self.message = None
        self.plural_message = None
        self.translated_message = None
        self.current_plural_index = 0
        self.plural_translated_messages.clear()

    def parse_po_filename(self, filename):
        # XXX: Actually, encoding may be different and specified in the .po file comment
        with io.open(filename, encoding='utf-8') as po_file:
            return self.parse_po_file(po_file, filename)

    def parse_po_file(self, fp, filename=None):
        for line_number, line in enumerate(fp):
            try:
                match = MATCHER.match(line)
                if match is None:
                    raise ParseError(filename, line_number, "Invalid syntax")
                if match.group(1):
                    # comment
                    continue
                if match.group(2) == 'msgctxt' and self.state not in ('after msgctxt', 'after msgid', 'after msgid_plural'):
                    self._store_pending_data()
                    if match.group(3) is not None:
                        raise ParseError(filename, line_number, "Unexpected index afted keyword")
                    if match.group(4) is None:
                        raise ParseError(filename, line_number, "Missing context string")
                    self.context = unescape(match.group(4))
                    self.state = 'after msgctxt'
                elif match.group(2) == 'msgid' and self.state not in ('after msgid', 'after msgid_plural'):
                    if self.state != 'after msgctxt':
                        self._store_pending_data()
                    if match.group(3) is not None:
                        raise ParseError(filename, line_number, "Unexpected index afted keyword")
                    if match.group(4) is None:
                        raise ParseError(filename, line_number, "Missing message identifier string")
                    self.message = unescape(match.group(4))
                    self.state = 'after msgid'
                elif match.group(2) == 'msgid_plural' and self.state == 'after msgid':
                    if match.group(3) is not None:
                        raise ParseError(filename, line_number, "Unexpected index after keyword")
                    if match.group(4) is None:
                        raise ParseError(filename, line_number, "Missing plural message identifier string")
                    self.plural_message = unescape(match.group(4))
                    self.state = 'after msgid_plural'
                elif match.group(2) == 'msgstr' and self.state in ('after msgid', 'after msgid_plural', 'after msgstr[N]'):
                    if match.group(4) is None:
                        raise ParseError(filename, line_number, "Missing translated message string")
                    if self.state == 'after msgid':
                        if match.group(3) is not None:
                            raise ParseError(filename, line_number, "Unexpected plural message index after keyword")
                        self.translated_message = unescape(match.group(4))
                        self.state = 'after msgstr'
                    elif self.state == 'after msgid_plural' or self.state == 'after msgstr[N]':
                        if match.group(3) is None:
                            raise ParseError(filename, line_number, "Missing plural message index after keyword")
                        self.current_plural_index = int(match.group(3))
                        if self.current_plural_index in self.plural_translated_messages:
                            raise ParseError(filename, line_number,
                                "Duplicate plural message index: {}".format(self.current_plural_index))
                        self.plural_translated_messages[self.current_plural_index] = unescape(match.group(4))
                        self.state = 'after msgstr[N]'
                elif match.group(2) is None:
                    if match.group(4) is None:
                        # empty-line
                        continue
                    if self.state == 'after msgid':
                        self.message += unescape(match.group(4))
                    elif self.state == 'after msgstr':
                        self.translated_message += unescape(match.group(4))
                    elif self.state == 'after msgid_plural':
                        self.plural_message += unescape(match.group(4))
                    elif self.state == 'after msgstr[N]':
                        self.plural_translated_messages[self.current_plural_index] += unescape(match.group(4))
                    else:
                        raise ParseError(filename, line_number,
                            "Unexpected string continuation after '{}'".format(self.state))
                else:
                    raise ParseError(filename, line_number, "Unexpected keyword: {}".format(match.group(2)))
            except UnescapeError as e:
                raise ParseError(filename, line_number, e)
        self._store_pending_data()
        if self.state in ('after msgctxt', 'after msgid', 'after msgid_plural'):
            raise ParseError(filename, None, "Unexpected end of file")


class ParseError(Exception):
    def __init__(self, filename, line_number, message, *args):
        self.filename = filename
        self.line_number = line_number
        self.parse_error_message = message
        super(ParseError, self).__init__(filename, line_number, message)

    def __str__(self):
        if self.filename and self.line_number:
            return "{}:{}: {}".format(self.filename, self.line_number, self.parse_error_message)
        elif self.filename:
            return "{}: {}".format(self.filename, self.parse_error_message)
        elif self.line_number:
            return "line {}: {}".format(self.line_number, self.parse_error_message)
        else:
            return self.parse_error_message



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import django
from django.conf import settings
from django.core import management

if not settings.configured:
    settings.configure()
    django_setup = getattr(django, 'setup', lambda: None)
    django_setup()

import io
import os
import shutil
import sys
import tempfile
import time
from django.test import SimpleTestCase
from django.utils import translation
from po_localization.tests.subtest import SubprocessTestCase


class IntegrationTestCase(SubprocessTestCase, SimpleTestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()
        shutil.copytree(os.path.join(os.path.dirname(__file__), 'test_app'), os.path.join(self.temp_dir, 'test_app'))
        shutil.copytree(os.path.join(os.path.dirname(__file__), 'test_project'), os.path.join(self.temp_dir, 'test_project'))
        sys.path.insert(0, self.temp_dir)
        self.locale_path = os.path.join(self.temp_dir, 'test_app/locale')

    def tearDown(self):
        sys.path.remove(self.temp_dir)
        shutil.rmtree(self.temp_dir)

    def test_simple(self):
        local_settings = {
            'ALLOWED_HOSTS': ['*'],
            'AUTO_RELOAD_TRANSLATIONS': True,
            'AUTO_UPDATE_TRANSLATIONS': True,
            'UPDATE_TRANSLATIONS_PACKAGES': (
                'test_app',
            ),
            'UPDATE_TRANSLATIONS_EXCLUDED_LOCALES': (
                'en',
            ),
            'MIDDLEWARE_CLASSES': (
                'po_localization.middleware.PoLocalizationMiddleware',
            ),
            'INSTALLED_APPS': (
                'po_localization',
                'test_app',
            ),
            'LANGUAGE_CODE': 'fr',
            'LANGUAGES': (
                ('fr', 'French'),
                ('en', 'English'),
            ),
            'ROOT_URLCONF': 'test_app.urls',
        }
        with self.settings(**local_settings):
            self.client.get('')
            self.assertTrue(os.path.exists(self.locale_path))
            french_translation_filename = os.path.join(self.locale_path, 'fr/LC_MESSAGES/django.po')
            self.assertTrue(os.path.isfile(french_translation_filename))
            english_translation_filename = os.path.join(self.locale_path, 'en/LC_MESSAGES/django.po')
            self.assertFalse(os.path.exists(english_translation_filename))

            with io.open(french_translation_filename, 'r', encoding='utf-8') as translation_file:
                self.assertEqual(
                    translation_file.read(),
"""#: models.py:12
msgid "test field"
msgstr ""

#: templates/index.html:8
msgid "test template"
msgstr ""

#: templates/index.html:10
msgid "%(counter)s item"
msgid_plural "%(counter)s items"
msgstr[0] ""
msgstr[1] ""

#: views.py:12
msgctxt "view context"
msgid "test view string"
msgstr ""
""")
            self.assertEqual("test field", translation.ugettext("test field"))
            # This sleep is required because mtimes do not have a high enough resolution
            time.sleep(1)
            with io.open(french_translation_filename, 'w', encoding='utf-8') as translation_file:
                translation_file.write(
"""
msgid "test field"
msgstr "champ de test"

msgid "test template"
msgstr "template de test"

msgid "%(counter)s item"
msgid_plural "%(counter)s items"
msgstr[0] "%(counter)s élément"
msgstr[1] "%(counter)s éléments"

msgctxt "view context"
msgid "test view string"
msgstr "chaîne de vue de test"
""")
            # No request triggered, still not translated
            self.assertEqual("test field", translation.ugettext("test field"))
            self.assertEqual("test template", translation.ugettext("test template"))
            self.assertEqual("test view string", translation.pgettext("view context", "test view string"))
            self.assertEqual("%(counter)s items", translation.ungettext("%(counter)s item", "%(counter)s items", 2))
            # Trigger a request, should reload translations
            self.assertEqual("chaîne de vue de test", self.client.get('').content.decode('utf-8'))
            self.assertEqual("champ de test", translation.ugettext("test field"))
            self.assertEqual("template de test", translation.ugettext("test template"))
            self.assertEqual("chaîne de vue de test", translation.pgettext("view context", "test view string"))
            self.assertEqual("%(counter)s éléments", translation.ungettext("%(counter)s item", "%(counter)s items", 2))

            # Test file update
            with io.open(os.path.join(self.temp_dir, 'test_app/models.py'), 'a', encoding='utf-8') as models_file:
                models_file.write('    second_field=models.TextField(verbose_name=_(\'second field\'))')
            self.client.get('')
            with io.open(french_translation_filename, 'r', encoding='utf-8') as translation_file:
                self.assertEqual(
                    translation_file.read(),
"""#: models.py:12
msgid "test field"
msgstr "champ de test"

#: models.py:13
msgid "second field"
msgstr ""

#: templates/index.html:8
msgid "test template"
msgstr "template de test"

#: templates/index.html:10
msgid "%(counter)s item"
msgid_plural "%(counter)s items"
msgstr[0] "%(counter)s élément"
msgstr[1] "%(counter)s éléments"

#: views.py:12
msgctxt "view context"
msgid "test view string"
msgstr "chaîne de vue de test"
""")
            # Test file removal
            os.unlink(os.path.join(self.temp_dir, 'test_app/views.py'))
            self.client.get('')
            with io.open(french_translation_filename, 'r', encoding='utf-8') as translation_file:
                self.assertEqual(
                    translation_file.read(),
"""#. obsolete entry
msgctxt "view context"
msgid "test view string"
msgstr "cha\xeene de vue de test"

#: models.py:12
msgid "test field"
msgstr "champ de test"

#: models.py:13
msgid "second field"
msgstr ""

#: templates/index.html:8
msgid "test template"
msgstr "template de test"

#: templates/index.html:10
msgid "%(counter)s item"
msgid_plural "%(counter)s items"
msgstr[0] "%(counter)s élément"
msgstr[1] "%(counter)s éléments"
""")
            with self.settings(UPDATE_TRANSLATIONS_PRUNE_OBSOLETES=True):
                self.client.get('')
                with io.open(french_translation_filename, 'r', encoding='utf-8') as translation_file:
                    self.assertEqual(
                        translation_file.read(),
"""#: models.py:12
msgid "test field"
msgstr "champ de test"

#: models.py:13
msgid "second field"
msgstr ""

#: templates/index.html:8
msgid "test template"
msgstr "template de test"

#: templates/index.html:10
msgid "%(counter)s item"
msgid_plural "%(counter)s items"
msgstr[0] "%(counter)s élément"
msgstr[1] "%(counter)s éléments"
""")

    def test_management_command(self):
        local_settings = {
            'AUTO_UPDATE_TRANSLATIONS': False,
            'INSTALLED_APPS': (
                'po_localization',
                'test_app',
            ),
            'MIDDLEWARE_CLASSES': (
                'po_localization.middleware.PoLocalizationMiddleware',
            ),
            'UPDATE_TRANSLATIONS_PACKAGES': (
                'test_app',
            ),
        }
        with self.settings(**local_settings):
            self.assertFalse(os.path.exists(self.locale_path))
            management.call_command('update_translations')
            self.assertTrue(os.path.exists(self.locale_path))

    def test_custom_locale_paths(self):
        local_settings = {
            'AUTO_RELOAD_TRANSLATIONS': True,
            'INSTALLED_APPS': (
                'po_localization',
            ),
            'MIDDLEWARE_CLASSES': (
                'po_localization.middleware.PoLocalizationMiddleware',
            ),
            'LOCALE_PATHS': (
                os.path.join(self.temp_dir, 'test_project/locale'),
            ),
            'LANGUAGE_CODE': 'fr',
            'LANGUAGES': (
                ('fr', 'French'),
                ('en', 'English'),
            ),
            'ROOT_URLCONF': 'test_project.urls'
        }
        with self.settings(**local_settings):
            response = self.client.get('')
            self.assertEqual("chaîne de test de projet", response.content.decode('utf-8'))



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import os
from unittest import TestCase
from po_localization.po_file import PoFile
from po_localization.template_extractor import extract_messages


class TemplateExtractorTestCase(TestCase):
    def setUp(self):
        po_file = PoFile()
        extract_messages(os.path.join(os.path.dirname(__file__), 'sample.html'), po_file)
        self.po_keys = po_file.entries.keys()

    def test_simple(self):
        self.assertIn("simple translation", self.po_keys)
        self.assertIn("translation to variable", self.po_keys)

    def test_context(self):
        self.assertIn("context\x04translation with context", self.po_keys)

    def test_block(self):
        self.assertIn("simple block translation", self.po_keys)
        self.assertIn("context\x04block translation with context", self.po_keys)

    def test_plural(self):
        self.assertIn("%(counter)s block translation", self.po_keys)
        self.assertIn("context\x04%(counter)s block translation with context", self.po_keys)

    def test_requires_i18n(self):
        self.assertNotIn("ignored translation before loading i18n", self.po_keys)

    def test_ignore_variable_translations(self):
        self.assertNotIn("ignored_variable_translation", self.po_keys)
        self.assertNotIn("ignored translation with variable context", self.po_keys)
        self.assertNotIn("variable_context\x04,ignored translation with variable context", self.po_keys)
        self.assertNotIn("ignored block translation with variable context", self.po_keys)
        self.assertNotIn("variable_context\x04,ignored block translation with variable context", self.po_keys)
        self.assertNotIn("ignored %(counter)s block translation with variable context", self.po_keys)
        self.assertNotIn("variable_context\x04ignored %(counter)s block translation with variable context", self.po_keys)

    def test_ignore_filtered_constants(self):
        self.assertNotIn("ignored translation with filter", self.po_keys)
        self.assertNotIn("ignored translation with filtered context", self.po_keys)
        self.assertNotIn("context\x04ignored translation with filtered context", self.po_keys)






# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from io import StringIO
from .strings import escape


class PoFile(object):
    def __init__(self):
        self.header_fields = []
        self._header_index = {}
        self.entries = {}

    def clone(self):
        po_file = PoFile()
        po_file.header_fields.extend(self.header_fields)
        for msgid, entry in self.entries.items():
            po_file.entries[msgid] = entry.clone()
        return po_file

    def add_header_field(self, field, value):
        if field in self._header_index:
            self.header_fields[self._header_index[field]] = (field, value)
        else:
            self._header_index[field] = len(self.header_fields)
            self.header_fields.append((field, value))

    def add_entry(self, message, plural=None, context=None):
        msgid = get_msgid(message, context)
        if msgid in self.entries:
            entry = self.entries[msgid]
            # Allow merging a non-plural entry with a plural entry
            # If more than one plural entry only keep the first
            if entry.plural is None:
                entry.plural = plural
        else:
            entry = TranslationEntry(message, plural, context)
            self.entries[msgid] = entry
        return entry

    def dump(self, fp, include_locations=True, prune_obsoletes=False):
        needs_blank_line = False
        if len(self.header_fields):
            print('msgid ""', file=fp)
            print('msgstr ""', file=fp)
            for field, value in self.header_fields:
                print(r'"{}: {}\n"'.format(field, value), file=fp)
            needs_blank_line = True
        nplurals = self.get_nplurals()
        for entry in sorted(self.entries.values(), key=get_entry_sort_key):
            if needs_blank_line:
                print('', file=fp)
            needs_blank_line = entry.dump(
                fp, nplurals, include_locations=include_locations, prune_obsolete=prune_obsoletes)

    def dumps(self, include_locations=True, prune_obsoletes=False):
        string_file = StringIO()
        self.dump(string_file, include_locations, prune_obsoletes)
        return string_file.getvalue()

    def get_catalog(self):
        catalog = {}
        for entry in self.entries.values():
            entry.fill_catalog(catalog)
        return catalog

    def get_nplurals(self):
        plural_field_index = self._header_index.get('Plural-Forms', -1)
        if plural_field_index != -1:
            field, value = self.header_fields[plural_field_index]
            if field == 'Plural-Forms':
                for pair in value.split(';'):
                    parts = pair.partition('=')
                    if parts[0].strip() == 'nplurals':
                        return int(parts[2].strip())
        return None


class TranslationEntry(object):
    def __init__(self, message, plural=None, context=None):
        self.message = message
        self.plural = plural
        self.context = context
        self.locations = []
        self.translations = {}

    def clone(self):
        entry = TranslationEntry(self.message, self.plural, self.context)
        entry.locations.extend(self.locations)
        entry.translations = self.translations.copy()
        return entry

    def add_location(self, filename, lineno):
        self.locations.append((filename, lineno))

    def add_translation(self, translation):
        self.add_plural_translation(0, translation)

    def add_plural_translation(self, index, translation):
        self.translations[index] = translation

    def fill_catalog(self, catalog):
        msgid = get_msgid(self.message, self.context)
        if self.plural is not None:
            for index, translation in self.translations.items():
                if translation:
                    catalog[(msgid, index)] = translation
        else:
            translation = self.translations.get(0, '')
            if translation:
                catalog[msgid] = translation

    def dump(self, fp, nplurals=None, min_nplurals=2, include_locations=True, prune_obsolete=False):
        """
        If plural, shows exactly 'nplurals' plurals if 'nplurals' is not None, else shows at least min_nplurals.
        All plural index are ordered and consecutive, missing entries are displayed with an empty string.
        """
        if not len(self.locations):
            if prune_obsolete or all(translation == '' for index, translation in self.translations.items()):
                return False
            else:
                print('#. obsolete entry', file=fp)
        if include_locations and len(self.locations):
            print('#: {}'.format(' '.join('{}:{}'.format(*location) for location in self.locations)), file=fp)
        if self.context is not None:
            print('msgctxt "{}"'.format(escape(self.context)), file=fp)
        print('msgid "{}"'.format(escape(self.message)), file=fp)
        if self.plural is not None:
            print('msgid_plural "{}"'.format(escape(self.plural)), file=fp)
            if nplurals is None:
                if len(self.translations) > 0:
                    nplurals = max(max(self.translations.keys()) + 1, min_nplurals)
                else:
                    nplurals = min_nplurals
            for index in range(nplurals):
                print('msgstr[{}] "{}"'.format(index, self.translations.get(index, '')), file=fp)
        else:
            print('msgstr "{}"'.format(self.translations.get(0, '')), file=fp)
        return True


def get_msgid(message, context=None):
    if context is not None:
        return '{}\x04{}'.format(context, message)
    else:
        return message

def get_entry_sort_key(entry):
    return entry.locations, entry.context if entry.context else '', entry.message



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import io
import os
from . import python_extractor, template_extractor
from .parser import Parser
from .po_file import PoFile

extractors = {
    '.html': template_extractor.extract_messages,
    '.txt': template_extractor.extract_messages,
    '.py': python_extractor.extract_messages
}
""":type extractors: dict[str, (str, str, str) -> None]"""


class TranslationsUpdater(object):
    def __init__(self, root_paths=(), locales=(), include_locations=True, prune_obsoletes=False):
        super(TranslationsUpdater, self).__init__()
        self.root_paths = root_paths
        self.locales = locales
        self.include_locations = include_locations
        self.prune_obsoletes = prune_obsoletes

    def execute(self):
        for root_path in self.root_paths:
            update_translations(
                root_path=root_path,
                locales=self.locales,
                include_locations=self.include_locations,
                prune_obsoletes=self.prune_obsoletes)

    def list_files(self):
        for root_path in self.root_paths:
            for dirpath, dirnames, filenames in os.walk(root_path):
                for filename in filenames:
                    extension = os.path.splitext(filename)[1]
                    if extension in extractors:
                        yield os.path.join(dirpath, filename)


def update_translations(
        root_path, domain='django', locales=(), locales_path='locale',
        update_all=True, include_locations=True, prune_obsoletes=False):
    base_po_file = PoFile()
    root_path_length = len(root_path) + (0 if root_path.endswith('/') else 1)
    locales_path = os.path.join(root_path, locales_path)
    for dirpath, dirnames, filenames in os.walk(root_path):
        for filename in filenames:
            extension = os.path.splitext(filename)[1]
            extractor = extractors.get(extension, None)
            if extractor is not None:
                full_filename = os.path.join(dirpath, filename)
                printable_filename = full_filename[root_path_length:]
                extractor(full_filename, base_po_file, printable_filename=printable_filename)
    for locale in locales:
        locale_path = os.path.join(locales_path, locale)
        if not os.path.exists(locale_path):
            os.makedirs(locale_path)
    if os.path.isdir(locales_path):
        for locale in os.listdir(locales_path):
            locale_path = os.path.join(locales_path, locale)
            if os.path.isdir(locale_path) and (update_all or locale in locales):
                locale_po_file = base_po_file.clone()
                translation_filename = os.path.join(locale_path, 'LC_MESSAGES/{}.po'.format(domain))
                if os.path.exists(translation_filename):
                    Parser(locale_po_file).parse_po_filename(translation_filename)
                # We do this dance to avoid overwriting the locale file with a broken one
                # if the dump function ever fails. A real atomic write may be better but
                # requires fiddling with file attributes and alters the inode (which is used
                # by osx aliases).
                memory_file = io.StringIO()
                locale_po_file.dump(memory_file, include_locations=include_locations, prune_obsoletes=prune_obsoletes)
                memory_file.seek(0)
                if not os.path.exists(os.path.dirname(translation_filename)):
                    os.makedirs(os.path.dirname(translation_filename))
                with io.open(translation_filename, 'w', encoding='utf-8') as locale_file:
                    locale_file.write(memory_file.read())



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import collections
import io
from django import template
from django.templatetags import i18n
from django.utils import six


def extract_messages(filename, po_file, printable_filename=None):
    printable_filename = filename if printable_filename is None else printable_filename
    with io.open(filename, 'r', encoding='utf-8') as sample_file:
        file_content = sample_file.read()
    lexer = template.Lexer(file_content, printable_filename)
    tokens = lexer.tokenize()
    parser = MessageParser(tokens)
    nodes = parser.parse()
    for node in nodes:
        if isinstance(node, LinenoNode):
            wrapped_node = node.wrapped_node
            if isinstance(wrapped_node, i18n.TranslateNode):
                message = resolve_literal_string_filter_expression(wrapped_node.filter_expression)
                if message is not None:
                    context, context_is_variable = get_context_and_variability(wrapped_node)
                    if not context_is_variable:
                        po_file.add_entry(message, context=context).add_location(lexer.origin, node.lineno)
            elif isinstance(wrapped_node, i18n.BlockTranslateNode):
                message, singular_vars = wrapped_node.render_token_list(wrapped_node.singular)
                context, context_is_variable = get_context_and_variability(wrapped_node)
                if not context_is_variable:
                    if wrapped_node.countervar and wrapped_node.counter:
                        plural, plural_vars = wrapped_node.render_token_list(wrapped_node.plural)
                        po_file.add_entry(message, plural, context).add_location(lexer.origin, node.lineno)
                    else:
                        po_file.add_entry(message, context=context).add_location(lexer.origin, node.lineno)


class MessageParser(template.Parser):
    def __init__(self, tokens):
        super(MessageParser, self).__init__(tokens)
        minimal_tags = collections.defaultdict(lambda: dummy_tag)
        minimal_tags['load'] = load_i18n_tag
        minimal_tags['comment'] = self.tags['comment']
        self.tags = minimal_tags

    def find_filter(self, filter_name):
        return None

    def compile_filter(self, token):
        return MessageFilterExpression(token, self)


class MessageFilterExpression(template.FilterExpression):
    @staticmethod
    def args_check(name, func, provided):
        return True

def load_i18n_tag(parser, token):
    if token.contents.split()[1] == 'i18n':
        original_library = template.import_library('django.templatetags.i18n')
        library = template.Library()
        library.tags['trans'] = lineno_tag(original_library.tags['trans'])
        library.tags['blocktrans'] = lineno_tag(original_library.tags['blocktrans'])
        parser.add_library(library)
    return template.Node()


def dummy_tag(parser, token):
    return template.Node()


def lineno_tag(wrapped_tag):
    def wrapper(parser, token):
        return LinenoNode(token.lineno, wrapped_tag(parser, token))

    return wrapper


class LinenoNode(template.Node):
    def __init__(self, lineno, wrapped_node):
        self.lineno = lineno
        self.wrapped_node = wrapped_node


def resolve_literal_string_filter_expression(expression):
    if expression is not None:
        if len(expression.filters) == 0:
            # Sometimes a string literal appears as a SafeText and sometime as a Variable with a literal
            # This may be related to the translate option of Variable
            if isinstance(expression.var, six.text_type):
                return expression.var
            if hasattr(expression.var, 'literal') and isinstance(expression.var.literal, six.text_type):
                return expression.var.literal


def get_context_and_variability(node):
    if node.message_context is not None:
        context = resolve_literal_string_filter_expression(node.message_context)
        return context, context is None
    return None, False



# coding=utf-8

from setuptools import setup

__version__ = 'unknown'
with open('po_localization/version.py') as version_file:
    exec(version_file.read())

with open('README.rst') as readme_file:
    long_description = readme_file.read()

setup(
    name='po_localization',
    packages=[
        'po_localization',
        'po_localization.management',
        'po_localization.management.commands',
        'po_localization.tests',
        'po_localization.tests.test_app',
        'po_localization.tests.test_project'],
    package_data={
        'po_localization.tests': ['*.html', '*.po'],
        'po_localization.tests.test_app': ['templates/*.html'],
        'po_localization.tests.test_project': ['locale/fr/LC_MESSAGES/*.po'],
    },
    version=__version__,
    description='Localize Django applications without compiling .po files',
    long_description=long_description,
    author='Kevin Michel',
    author_email='kmichel.info@gmail.com',
    url='https://github.com/kmichel/po-localization',
    download_url='https://github.com/kmichel/po-localization/archive/v{}.tar.gz'.format(__version__),
    keywords=['django', 'localization'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Localization',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=['django>=1.6'],
)



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import os
import django.utils.translation.trans_real
from . import parser


class TranslationsLoader(object):
    def __init__(self, locale_paths=(), locales=()):
        self.locale_paths = locale_paths
        self.locales = locales
        super(TranslationsLoader, self).__init__()

    def execute(self):
        for locale in self.locales:
            language = django.utils.translation.trans_real.to_language(locale)
            # XXX: this will trigger useless loading of translations from django code
            catalog = django.utils.translation.trans_real.translation(language)._catalog
            catalog.clear()
            for file_path in self._get_translation_files(locale):
                catalog.update(parser.parse_po_filename(file_path))

    def list_files(self):
        for locale in self.locales:
            for file_path in self._get_translation_files(locale):
                yield file_path

    def _get_translation_files(self, locale):
        for locale_path in self.locale_paths:
            translation_path = os.path.join(locale_path, locale, 'LC_MESSAGES/django.po')
            if os.path.isfile(translation_path):
                yield translation_path



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _


class TestModel(models.Model):
    test_field = models.TextField(verbose_name=_('test field'))



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import base64
import os
import pickle
import subprocess
import sys
from unittest import TestCase, TestResult


class SubprocessTestCase(TestCase):
    def __call__(self, result=None):
        if result is not None and isinstance(result, PicklableTestResult):
            super(SubprocessTestCase, self).__call__(result)
        else:
            orig_result = result
            if result is None:
                result = self.defaultTestResult()
                start_test_run = getattr(result, 'startTestRun', None)
                if start_test_run is not None and callable(start_test_run):
                    start_test_run()

            self._resultForDoCleanups = result
            result.startTest(self)
            try:
                test_module_name = self.__class__.__module__
                test_class_name = self.__class__.__name__
                test_method_name = self._testMethodName
                subtest_thunk = os.path.join(os.path.dirname(__file__), 'subtest_thunk.py')
                args = [sys.executable, subtest_thunk, test_module_name, test_class_name, test_method_name]
                environment = os.environ
                if 'coverage' in sys.modules:
                    environment = environment.copy()
                    environment['COVERAGE_PROCESS_START'] = '.coveragerc'
                proc = subprocess.Popen(args, stdout=subprocess.PIPE, env=environment)
                stdout_content, nothing = proc.communicate()
                if proc.returncode != 0:
                    print_bytes(stdout_content)
                    raise subprocess.CalledProcessError(proc.returncode, args)
                else:
                    user_stdout_content, separator, encoded_data = stdout_content.rpartition(b'-')
                    print_bytes(user_stdout_content)
                    decoded_data = base64.decodestring(encoded_data)
                    result_actions = pickle.loads(decoded_data)
                    for action, arguments in result_actions:
                        getattr(result, action)(*arguments)
            finally:
                result.stopTest(self)
                if orig_result is None:
                    stop_test_run = getattr(result, 'stopTestRun', None)
                    if stop_test_run is not None and callable(stop_test_run):
                        stop_test_run()


def print_bytes(content):
    output = sys.stdout.buffer if hasattr(sys.stdout, 'buffer') else sys.stdout
    sys.stdout.flush()
    output.write(content)
    output.flush()


class PicklableTestResult(TestResult):
    def __init__(self, *args, **kwargs):
        super(PicklableTestResult, self).__init__(*args, **kwargs)
        self.storage = []

    def addError(self, test, err):
        self.storage.append(('addError', (PicklableTest(test), get_picklable_exc_info(err))))

    def addExpectedFailure(self, test, err):
        self.storage.append(('addExpectedFailure', (PicklableTest(test), get_picklable_exc_info(err))))

    def addFailure(self, test, err):
        self.storage.append(('addFailure', (PicklableTest(test), get_picklable_exc_info(err))))

    def addSkip(self, test, reason):
        self.storage.append(('addSkip', (PicklableTest(test), reason)))

    def addSuccess(self, test):
        self.storage.append(('addSuccess', (PicklableTest(test),)))

    def addUnexpectedSuccess(self, test):
        self.storage.append(('addUnexpectedSuccess', (PicklableTest(test),)))


class PicklableTest(object):
    def __init__(self, test):
        self.failureException = test.failureException
        self._testMethodName = getattr(test, '_testMethodName', None)
        self._shortDescription = test.shortDescription()
        self._str = test.__str__()
        self._repr = test.__repr__()

    def shortDescription(self):
        return self._shortDescription

    def __str__(self):
        return self._str

    def __repr__(self):
        return self._repr


def get_picklable_exc_info(exc_info):
    exception_type, exception_value, traceback = exc_info
    return exception_type, exception_value, PicklableTraceback(traceback)


class PicklableTraceback(object):
    def __init__(self, traceback):
        self.tb_frame = PicklableTracebackFrame(traceback.tb_frame)
        self.tb_lineno = traceback.tb_lineno
        self.tb_next = None if traceback.tb_next is None else PicklableTraceback(traceback.tb_next)


class PicklableTracebackFrame(object):
    def __init__(self, traceback_frame):
        self.f_code = PicklableCode(traceback_frame.f_code)
        self.f_globals = dict((key, None) for key in traceback_frame.f_globals.keys())


class PicklableCode(object):
    def __init__(self, code):
        self.co_filename = code.co_filename
        self.co_name = code.co_name



# coding=utf-8

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import os
from unittest import TestCase
from po_localization.po_file import PoFile
from po_localization.python_extractor import extract_messages


class PythonExtractorTestCase(TestCase):
    def test_full_module(self):
        po_file = PoFile()
        extract_messages(os.path.join(os.path.dirname(__file__), 'sample.py'), po_file, 'sample.py')
        po_keys = po_file.entries.keys()
        self.assertIn('test', po_keys)
        self.assertIn('test_lazy', po_keys)
        self.assertIn('test_noop', po_keys)
        self.assertIn('utest', po_keys)
        self.assertIn('utest_lazy', po_keys)
        self.assertIn('utest_noop', po_keys)
        self.assertIn('context\x04ptest', po_keys)
        self.assertIn('context\x04ptest_lazy', po_keys)
        self.assertIn('context\x04ptest', po_keys)
        self.assertIn('ntest', po_keys)
        self.assertIn('ntest_lazy', po_keys)
        self.assertIn('untest', po_keys)
        self.assertIn('untest_lazy', po_keys)
        self.assertIn('context\x04nptest', po_keys)
        self.assertIn('context\x04nptest_lazy', po_keys)

    def test_module_alias(self):
        po_file = PoFile()
        extract_messages(os.path.join(os.path.dirname(__file__), 'sample.py'), po_file, 'sample.py')
        po_keys = po_file.entries.keys()
        self.assertIn('test', po_keys)
        self.assertIn('test_lazy', po_keys)
        self.assertIn('test_noop', po_keys)
        self.assertIn('utest', po_keys)
        self.assertIn('utest_lazy', po_keys)
        self.assertIn('utest_noop', po_keys)
        self.assertIn('context\x04ptest', po_keys)
        self.assertIn('context\x04ptest_lazy', po_keys)
        self.assertIn('context\x04ptest', po_keys)
        self.assertIn('ntest', po_keys)
        self.assertIn('ntest_lazy', po_keys)
        self.assertIn('untest', po_keys)
        self.assertIn('untest_lazy', po_keys)
        self.assertIn('context\x04nptest', po_keys)
        self.assertIn('context\x04nptest_lazy', po_keys)

    def test_module_alias_2(self):
        po_file = PoFile()
        extract_messages(os.path.join(os.path.dirname(__file__), 'sample.py'), po_file, 'sample.py')
        po_keys = po_file.entries.keys()
        self.assertIn('tm2_test', po_keys)
        self.assertIn('tm2_test_lazy', po_keys)
        self.assertIn('tm2_test_noop', po_keys)
        self.assertIn('tm2_utest', po_keys)
        self.assertIn('tm2_utest_lazy', po_keys)
        self.assertIn('tm2_utest_noop', po_keys)
        self.assertIn('context\x04tm2_ptest', po_keys)
        self.assertIn('context\x04tm2_ptest_lazy', po_keys)
        self.assertIn('context\x04tm2_ptest', po_keys)
        self.assertIn('tm2_ntest', po_keys)
        self.assertIn('tm2_ntest_lazy', po_keys)
        self.assertIn('tm2_untest', po_keys)
        self.assertIn('tm2_untest_lazy', po_keys)
        self.assertIn('context\x04tm2_nptest', po_keys)
        self.assertIn('context\x04tm2_nptest_lazy', po_keys)

    def test_function_aliases(self):
        po_file = PoFile()
        extract_messages(os.path.join(os.path.dirname(__file__), 'sample.py'), po_file, 'sample.py')
        po_keys = po_file.entries.keys()
        self.assertIn('alias_test', po_keys)
        self.assertIn('alias_test_lazy', po_keys)
        self.assertIn('alias_test_noop', po_keys)
        self.assertIn('alias_utest', po_keys)
        self.assertIn('alias_utest_lazy', po_keys)
        self.assertIn('alias_utest_noop', po_keys)
        self.assertIn('context\x04alias_ptest', po_keys)
        self.assertIn('context\x04alias_ptest_lazy', po_keys)
        self.assertIn('context\x04alias_ptest', po_keys)
        self.assertIn('alias_ntest', po_keys)
        self.assertIn('alias_ntest_lazy', po_keys)
        self.assertIn('alias_untest', po_keys)
        self.assertIn('alias_untest_lazy', po_keys)
        self.assertIn('context\x04alias_nptest', po_keys)
        self.assertIn('context\x04alias_nptest_lazy', po_keys)

    def test_scoping(self):
        po_file = PoFile()
        extract_messages(os.path.join(os.path.dirname(__file__), 'sample.py'), po_file, 'sample.py')
        po_keys = po_file.entries.keys()
        self.assertNotIn('not a translation', po_keys)
        self.assertIn('a translation', po_keys)
        self.assertNotIn('still not a translation', po_keys)
